"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var utils_exports = {};
__export(utils_exports, {
  createRouteHandler: () => createRouteHandler,
  isNormalMethod: () => isNormalMethod
});
module.exports = __toCommonJS(utils_exports);
var import_reflect_metadata = require("reflect-metadata");
var import_bff_core = require("@modern-js/bff-core");
var import_bff_runtime = require("@modern-js/bff-runtime");
var import_type_is = __toESM(require("type-is"));
var import_formidable = __toESM(require("formidable"));
const handleResponseMeta = (res, handler) => {
  const responseMeta = Reflect.getMetadata(import_bff_core.HttpMetadata.Response, handler);
  if (Array.isArray(responseMeta)) {
    for (const meta of responseMeta) {
      const metaType = meta.type;
      const metaValue = meta.value;
      switch (metaType) {
        case import_bff_core.ResponseMetaType.Headers:
          for (const [key, value] of Object.entries(metaValue)) {
            res.append(key, value);
          }
          break;
        case import_bff_core.ResponseMetaType.Redirect:
          res.redirect(metaValue);
          break;
        case import_bff_core.ResponseMetaType.StatusCode:
          res.status(metaValue);
          break;
        default:
          break;
      }
    }
  }
};
const createRouteHandler = (handler) => {
  const apiHandler = async (req, res, next) => {
    const input = await getInputFromRequest(req);
    if ((0, import_bff_core.isWithMetaHandler)(handler)) {
      try {
        handleResponseMeta(res, handler);
        if (res.headersSent) {
          return;
        }
        const result = await handler(input);
        if (result && typeof result === "object") {
          return res.json(result);
        }
      } catch (error) {
        if (error instanceof import_bff_core.ValidationError) {
          res.status(error.status);
          return res.json({
            message: error.message
          });
        }
        throw error;
      }
    } else if ((0, import_bff_runtime.isSchemaHandler)(handler)) {
      const result = await handler(input);
      if (result.type !== "HandleSuccess") {
        if (result.type === "InputValidationError") {
          res.status(400);
        } else {
          res.status(500);
        }
        return res.json(result.message);
      } else {
        res.status(200);
        return res.json(result.value);
      }
    } else if ((0, import_bff_core.isInputParamsDeciderHandler)(handler)) {
      try {
        var _input_data;
        const args = (input === null || input === void 0 ? void 0 : (_input_data = input.data) === null || _input_data === void 0 ? void 0 : _input_data.args) || [];
        const body = await handler(...args);
        if (typeof body !== "undefined") {
          if (typeof body === "object") {
            return res.json(body);
          }
          return res.send(body);
        }
      } catch (e) {
        return next(e);
      }
    } else {
      const args = Object.values(input.params).concat(input);
      try {
        const body = await handler(...args);
        if (res.headersSent) {
          return await Promise.resolve();
        }
        if (typeof body !== "undefined") {
          return res.json(body);
        }
      } catch (e) {
        return next(e);
      }
    }
  };
  Object.defineProperties(apiHandler, Object.getOwnPropertyDescriptors(handler));
  return apiHandler;
};
const isNormalMethod = (httpMethod) => import_bff_core.httpMethods.includes(httpMethod);
const getInputFromRequest = async (request) => {
  const draft = {
    params: request.params,
    query: request.query,
    headers: request.headers,
    cookies: request.headers.cookie
  };
  if ((0, import_type_is.default)(request, [
    "application/json"
  ])) {
    draft.data = request.body;
  } else if ((0, import_type_is.default)(request, [
    "multipart/form-data"
  ])) {
    draft.formData = await resolveFormData(request);
  } else if ((0, import_type_is.default)(request, [
    "application/x-www-form-urlencoded"
  ])) {
    draft.formUrlencoded = request.body;
  } else {
    draft.body = request.body;
  }
  return draft;
};
const resolveFormData = (request) => {
  const form = (0, import_formidable.default)({
    multiples: true
  });
  return new Promise((resolve, reject) => {
    form.parse(request, (err, fields, files) => {
      if (err) {
        reject(err);
      }
      resolve({
        ...fields,
        ...files
      });
    });
  });
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createRouteHandler,
  isNormalMethod
});
